@@ -52,9 +52,8 @@ struct Decimal;
 // ----------------------------------------------------------------------
 // Generic array data container
 
-namespace internal {
-
-/// \brief Mutable internal container for generic Arrow array data
+/// \class ArrayData
+/// \brief Mutable container for generic Arrow array data
 ///
 /// This data structure is a self-contained representation of the memory and
 /// metadata inside an Arrow array data structure (called vectors in Java). The
@@ -88,47 +87,47 @@ struct ARROW_EXPORT ArrayData {
   ArrayData() {}
 
   ArrayData(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::vector<std::shared_ptr<Buffer>>& buffers,
-      int64_t null_count = kUnknownNullCount, int64_t offset = 0)
-      : type(type),
-        length(length),
-        buffers(buffers),
-        null_count(null_count),
-        offset(offset) {}
+            int64_t null_count = kUnknownNullCount, int64_t offset = 0)
+      : type(type), length(length), null_count(null_count), offset(offset) {}
 
   ArrayData(const std::shared_ptr<DataType>& type, int64_t length,
-      std::vector<std::shared_ptr<Buffer>>&& buffers,
-      int64_t null_count = kUnknownNullCount, int64_t offset = 0)
-      : type(type),
-        length(length),
-        buffers(std::move(buffers)),
-        null_count(null_count),
-        offset(offset) {}
+            const std::vector<std::shared_ptr<Buffer>>& buffers,
+            int64_t null_count = kUnknownNullCount, int64_t offset = 0)
+      : ArrayData(type, length, null_count, offset) {
+    this->buffers = buffers;
+  }
+
+  ArrayData(const std::shared_ptr<DataType>& type, int64_t length,
+            std::vector<std::shared_ptr<Buffer>>&& buffers,
+            int64_t null_count = kUnknownNullCount, int64_t offset = 0)
+      : ArrayData(type, length, null_count, offset) {
+    this->buffers = std::move(buffers);
+  }
 
   // Move constructor
   ArrayData(ArrayData&& other) noexcept
       : type(std::move(other.type)),
         length(other.length),
-        buffers(std::move(other.buffers)),
         null_count(other.null_count),
         offset(other.offset),
+        buffers(std::move(other.buffers)),
         child_data(std::move(other.child_data)) {}
 
   ArrayData(const ArrayData& other) noexcept
       : type(other.type),
         length(other.length),
-        buffers(other.buffers),
         null_count(other.null_count),
         offset(other.offset),
+        buffers(other.buffers),
         child_data(other.child_data) {}
 
   // Move assignment
   ArrayData& operator=(ArrayData&& other) {
     type = std::move(other.type);
     length = other.length;
-    buffers = std::move(other.buffers);
     null_count = other.null_count;
     offset = other.offset;
+    buffers = std::move(other.buffers);
     child_data = std::move(other.child_data);
     return *this;
   }
@@ -139,16 +138,18 @@ struct ARROW_EXPORT ArrayData {
 
   std::shared_ptr<DataType> type;
   int64_t length;
-  std::vector<std::shared_ptr<Buffer>> buffers;
   int64_t null_count;
   int64_t offset;
+  std::vector<std::shared_ptr<Buffer>> buffers;
   std::vector<std::shared_ptr<ArrayData>> child_data;
 };
 
-Status ARROW_EXPORT MakeArray(
-    const std::shared_ptr<ArrayData>& data, std::shared_ptr<Array>* out);
-
-}  // namespace internal
+/// \brief Create a strongly-typed Array instance from generic ArrayData
+/// \param[in] data the array contents
+/// \param[out] out the resulting Array instance
+/// \return Status
+ARROW_EXPORT
+Status MakeArray(const std::shared_ptr<ArrayData>& data, std::shared_ptr<Array>* out);
 
 // ----------------------------------------------------------------------
 // User array accessor types
@@ -211,39 +212,42 @@ class ARROW_EXPORT Array {
   /// Compare if the range of slots specified are equal for the given array and
   /// this array.  end_idx exclusive.  This methods does not bounds check.
   bool RangeEquals(int64_t start_idx, int64_t end_idx, int64_t other_start_idx,
-      const std::shared_ptr<Array>& other) const;
+                   const std::shared_ptr<Array>& other) const;
 
   bool RangeEquals(const Array& other, int64_t start_idx, int64_t end_idx,
-      int64_t other_start_idx) const;
+                   int64_t other_start_idx) const;
 
   Status Accept(ArrayVisitor* visitor) const;
 
   /// Construct a zero-copy slice of the array with the indicated offset and
   /// length
   ///
-  /// \param[in] offset the position of the first element in the constructed slice
-  /// \param[in] length the length of the slice. If there are not enough elements in the
-  /// array,
-  ///     the length will be adjusted accordingly
+  /// \param[in] offset the position of the first element in the constructed
+  /// slice
+  /// \param[in] length the length of the slice. If there are not enough
+  /// elements in the array, the length will be adjusted accordingly
   ///
   /// \return a new object wrapped in std::shared_ptr<Array>
-  virtual std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const = 0;
+  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const;
 
   /// Slice from offset until end of the array
   std::shared_ptr<Array> Slice(int64_t offset) const;
 
-  std::shared_ptr<internal::ArrayData> data() const { return data_; }
+  std::shared_ptr<ArrayData> data() const { return data_; }
 
   int num_fields() const { return static_cast<int>(data_->child_data.size()); }
 
+  /// \return PrettyPrint representation of array suitable for debugging
+  std::string ToString() const;
+
  protected:
   Array() {}
 
-  std::shared_ptr<internal::ArrayData> data_;
+  std::shared_ptr<ArrayData> data_;
   const uint8_t* null_bitmap_data_;
 
   /// Protected method for constructors
-  inline void SetData(const std::shared_ptr<internal::ArrayData>& data) {
+  inline void SetData(const std::shared_ptr<ArrayData>& data) {
     if (data->buffers.size() > 0 && data->buffers[0]) {
       null_bitmap_data_ = data->buffers[0]->data();
     } else {
@@ -253,10 +257,13 @@ class ARROW_EXPORT Array {
   }
 
  private:
-  DISALLOW_COPY_AND_ASSIGN(Array);
+  ARROW_DISALLOW_COPY_AND_ASSIGN(Array);
 };
 
-ARROW_EXPORT std::ostream& operator<<(std::ostream& os, const Array& x);
+static inline std::ostream& operator<<(std::ostream& os, const Array& x) {
+  os << x.ToString();
+  return os;
+}
 
 class ARROW_EXPORT FlatArray : public Array {
  protected:
@@ -268,37 +275,41 @@ class ARROW_EXPORT NullArray : public FlatArray {
  public:
   using TypeClass = NullType;
 
-  explicit NullArray(const std::shared_ptr<internal::ArrayData>& data) { SetData(data); }
-
+  explicit NullArray(const std::shared_ptr<ArrayData>& data) { SetData(data); }
   explicit NullArray(int64_t length);
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
+ private:
+  inline void SetData(const std::shared_ptr<ArrayData>& data) {
+    null_bitmap_data_ = nullptr;
+    data->null_count = data->length;
+    data_ = data;
+  }
 };
 
 /// Base class for fixed-size logical types
 class ARROW_EXPORT PrimitiveArray : public FlatArray {
  public:
   PrimitiveArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+                 const std::shared_ptr<Buffer>& data,
+                 const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+                 int64_t null_count = 0, int64_t offset = 0);
 
   /// Does not account for any slice offset
   std::shared_ptr<Buffer> values() const { return data_->buffers[1]; }
 
-  /// Does not account for any slice offset
-  const uint8_t* raw_values() const { return raw_values_; }
+  /// \brief Return pointer to start of raw data
+  const uint8_t* raw_values() const;
 
  protected:
   PrimitiveArray() {}
 
-  inline void SetData(const std::shared_ptr<internal::ArrayData>& data) {
+  inline void SetData(const std::shared_ptr<ArrayData>& data) {
     auto values = data->buffers[1];
     this->Array::SetData(data);
     raw_values_ = values == nullptr ? nullptr : values->data();
   }
 
-  explicit inline PrimitiveArray(const std::shared_ptr<internal::ArrayData>& data) {
+  explicit inline PrimitiveArray(const std::shared_ptr<ArrayData>& data) {
     SetData(data);
   }
 
@@ -311,7 +322,7 @@ class ARROW_EXPORT NumericArray : public PrimitiveArray {
   using TypeClass = TYPE;
   using value_type = typename TypeClass::c_type;
 
-  explicit NumericArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit NumericArray(const std::shared_ptr<ArrayData>& data);
 
   // Only enable this constructor without a type argument for types without additional
   // metadata
@@ -322,14 +333,12 @@ class ARROW_EXPORT NumericArray : public PrimitiveArray {
       const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
       int64_t offset = 0)
       : PrimitiveArray(TypeTraits<T1>::type_singleton(), length, data, null_bitmap,
-            null_count, offset) {}
+                       null_count, offset) {}
 
   const value_type* raw_values() const {
     return reinterpret_cast<const value_type*>(raw_values_) + data_->offset;
   }
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
   value_type Value(int64_t i) const { return raw_values()[i]; }
 
  protected:
@@ -340,17 +349,15 @@ class ARROW_EXPORT BooleanArray : public PrimitiveArray {
  public:
   using TypeClass = BooleanType;
 
-  explicit BooleanArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit BooleanArray(const std::shared_ptr<ArrayData>& data);
 
   BooleanArray(int64_t length, const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
-
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
+               const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+               int64_t null_count = 0, int64_t offset = 0);
 
   bool Value(int64_t i) const {
-    return BitUtil::GetBit(
-        reinterpret_cast<const uint8_t*>(raw_values_), i + data_->offset);
+    return BitUtil::GetBit(reinterpret_cast<const uint8_t*>(raw_values_),
+                           i + data_->offset);
   }
 
  protected:
@@ -364,12 +371,13 @@ class ARROW_EXPORT ListArray : public Array {
  public:
   using TypeClass = ListType;
 
-  explicit ListArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit ListArray(const std::shared_ptr<ArrayData>& data);
 
   ListArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::shared_ptr<Buffer>& value_offsets, const std::shared_ptr<Array>& values,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+            const std::shared_ptr<Buffer>& value_offsets,
+            const std::shared_ptr<Array>& values,
+            const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
+            int64_t offset = 0);
 
   /// \brief Construct ListArray from array of offsets and child value array
   ///
@@ -382,7 +390,7 @@ class ARROW_EXPORT ListArray : public Array {
   /// allocated because of null values
   /// \param[out] out Will have length equal to offsets.length() - 1
   static Status FromArrays(const Array& offsets, const Array& values, MemoryPool* pool,
-      std::shared_ptr<Array>* out);
+                           std::shared_ptr<Array>* out);
 
   /// \brief Return array object containing the list's values
   std::shared_ptr<Array> values() const;
@@ -402,10 +410,8 @@ class ARROW_EXPORT ListArray : public Array {
     return raw_value_offsets_[i + 1] - raw_value_offsets_[i];
   }
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
  protected:
-  void SetData(const std::shared_ptr<internal::ArrayData>& data);
+  void SetData(const std::shared_ptr<ArrayData>& data);
   const int32_t* raw_value_offsets_;
 
  private:
@@ -419,12 +425,12 @@ class ARROW_EXPORT BinaryArray : public FlatArray {
  public:
   using TypeClass = BinaryType;
 
-  explicit BinaryArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit BinaryArray(const std::shared_ptr<ArrayData>& data);
 
   BinaryArray(int64_t length, const std::shared_ptr<Buffer>& value_offsets,
-      const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+              const std::shared_ptr<Buffer>& data,
+              const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+              int64_t null_count = 0, int64_t offset = 0);
 
   // Return the pointer to the given elements bytes
   // TODO(emkornfield) introduce a StringPiece or something similar to capture zero-copy
@@ -453,21 +459,20 @@ class ARROW_EXPORT BinaryArray : public FlatArray {
     return raw_value_offsets_[i + 1] - raw_value_offsets_[i];
   }
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
  protected:
   // For subclasses
   BinaryArray() {}
 
   /// Protected method for constructors
-  void SetData(const std::shared_ptr<internal::ArrayData>& data);
+  void SetData(const std::shared_ptr<ArrayData>& data);
 
   // Constructor that allows sub-classes/builders to propagate there logical type up the
   // class hierarchy.
   BinaryArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::shared_ptr<Buffer>& value_offsets, const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+              const std::shared_ptr<Buffer>& value_offsets,
+              const std::shared_ptr<Buffer>& data,
+              const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+              int64_t null_count = 0, int64_t offset = 0);
 
   const int32_t* raw_value_offsets_;
   const uint8_t* raw_data_;
@@ -477,12 +482,12 @@ class ARROW_EXPORT StringArray : public BinaryArray {
  public:
   using TypeClass = StringType;
 
-  explicit StringArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit StringArray(const std::shared_ptr<ArrayData>& data);
 
   StringArray(int64_t length, const std::shared_ptr<Buffer>& value_offsets,
-      const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+              const std::shared_ptr<Buffer>& data,
+              const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+              int64_t null_count = 0, int64_t offset = 0);
 
   // Construct a std::string
   // TODO: std::bad_alloc possibility
@@ -491,8 +496,6 @@ class ARROW_EXPORT StringArray : public BinaryArray {
     const uint8_t* str = GetValue(i, &nchars);
     return std::string(reinterpret_cast<const char*>(str), nchars);
   }
-
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
 };
 
 // ----------------------------------------------------------------------
@@ -502,23 +505,20 @@ class ARROW_EXPORT FixedSizeBinaryArray : public PrimitiveArray {
  public:
   using TypeClass = FixedSizeBinaryType;
 
-  explicit FixedSizeBinaryArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit FixedSizeBinaryArray(const std::shared_ptr<ArrayData>& data);
 
   FixedSizeBinaryArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+                       const std::shared_ptr<Buffer>& data,
+                       const std::shared_ptr<Buffer>& null_bitmap = nullptr,
+                       int64_t null_count = 0, int64_t offset = 0);
 
   const uint8_t* GetValue(int64_t i) const;
+  const uint8_t* Value(int64_t i) const { return GetValue(i); }
 
   int32_t byte_width() const { return byte_width_; }
 
-  const uint8_t* raw_values() const { return raw_values_; }
-
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
  protected:
-  inline void SetData(const std::shared_ptr<internal::ArrayData>& data) {
+  inline void SetData(const std::shared_ptr<ArrayData>& data) {
     this->PrimitiveArray::SetData(data);
     byte_width_ = static_cast<const FixedSizeBinaryType&>(*type()).byte_width();
   }
@@ -528,42 +528,16 @@ class ARROW_EXPORT FixedSizeBinaryArray : public PrimitiveArray {
 
 // ----------------------------------------------------------------------
 // DecimalArray
-class ARROW_EXPORT DecimalArray : public FlatArray {
+class ARROW_EXPORT DecimalArray : public FixedSizeBinaryArray {
  public:
-  using TypeClass = Type;
+  using TypeClass = DecimalType;
 
-  /// \brief Construct DecimalArray from internal::ArrayData instance
-  explicit DecimalArray(const std::shared_ptr<internal::ArrayData>& data);
+  using FixedSizeBinaryArray::FixedSizeBinaryArray;
 
-  DecimalArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::shared_ptr<Buffer>& data,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0, const std::shared_ptr<Buffer>& sign_bitmap = nullptr);
-
-  bool IsNegative(int64_t i) const;
-
-  const uint8_t* GetValue(int64_t i) const;
+  /// \brief Construct DecimalArray from ArrayData instance
+  explicit DecimalArray(const std::shared_ptr<ArrayData>& data);
 
   std::string FormatValue(int64_t i) const;
-
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
-  /// \brief The main decimal data
-  /// For 32/64-bit decimal this is everything
-  std::shared_ptr<Buffer> values() const { return data_->buffers[1]; }
-
-  /// Only needed for 128 bit Decimals
-  std::shared_ptr<Buffer> sign_bitmap() const { return data_->buffers[2]; }
-
-  int32_t byte_width() const {
-    return static_cast<const DecimalType&>(*type()).byte_width();
-  }
-  const uint8_t* raw_values() const { return raw_values_; }
-
- private:
-  void SetData(const std::shared_ptr<internal::ArrayData>& data);
-  const uint8_t* raw_values_;
-  const uint8_t* sign_bitmap_data_;
 };
 
 // ----------------------------------------------------------------------
@@ -573,18 +547,20 @@ class ARROW_EXPORT StructArray : public Array {
  public:
   using TypeClass = StructType;
 
-  explicit StructArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit StructArray(const std::shared_ptr<ArrayData>& data);
 
   StructArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::vector<std::shared_ptr<Array>>& children,
-      std::shared_ptr<Buffer> null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+              const std::vector<std::shared_ptr<Array>>& children,
+              std::shared_ptr<Buffer> null_bitmap = nullptr, int64_t null_count = 0,
+              int64_t offset = 0);
 
   // Return a shared pointer in case the requestor desires to share ownership
   // with this array.
   std::shared_ptr<Array> field(int pos) const;
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
+ private:
+  // For caching boxed child data
+  mutable std::vector<std::shared_ptr<Array>> boxed_fields_;
 };
 
 // ----------------------------------------------------------------------
@@ -595,14 +571,14 @@ class ARROW_EXPORT UnionArray : public Array {
   using TypeClass = UnionType;
   using type_id_t = uint8_t;
 
-  explicit UnionArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit UnionArray(const std::shared_ptr<ArrayData>& data);
 
   UnionArray(const std::shared_ptr<DataType>& type, int64_t length,
-      const std::vector<std::shared_ptr<Array>>& children,
-      const std::shared_ptr<Buffer>& type_ids,
-      const std::shared_ptr<Buffer>& value_offsets = nullptr,
-      const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
-      int64_t offset = 0);
+             const std::vector<std::shared_ptr<Array>>& children,
+             const std::shared_ptr<Buffer>& type_ids,
+             const std::shared_ptr<Buffer>& value_offsets = nullptr,
+             const std::shared_ptr<Buffer>& null_bitmap = nullptr, int64_t null_count = 0,
+             int64_t offset = 0);
 
   /// Note that this buffer does not account for any slice offset
   std::shared_ptr<Buffer> type_ids() const { return data_->buffers[1]; }
@@ -617,13 +593,14 @@ class ARROW_EXPORT UnionArray : public Array {
 
   std::shared_ptr<Array> child(int pos) const;
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
  protected:
-  void SetData(const std::shared_ptr<internal::ArrayData>& data);
+  void SetData(const std::shared_ptr<ArrayData>& data);
 
   const type_id_t* raw_type_ids_;
   const int32_t* raw_value_offsets_;
+
+  // For caching boxed child data
+  mutable std::vector<std::shared_ptr<Array>> boxed_fields_;
 };
 
 // ----------------------------------------------------------------------
@@ -648,20 +625,18 @@ class ARROW_EXPORT DictionaryArray : public Array {
  public:
   using TypeClass = DictionaryType;
 
-  explicit DictionaryArray(const std::shared_ptr<internal::ArrayData>& data);
+  explicit DictionaryArray(const std::shared_ptr<ArrayData>& data);
 
-  DictionaryArray(
-      const std::shared_ptr<DataType>& type, const std::shared_ptr<Array>& indices);
+  DictionaryArray(const std::shared_ptr<DataType>& type,
+                  const std::shared_ptr<Array>& indices);
 
   std::shared_ptr<Array> indices() const;
   std::shared_ptr<Array> dictionary() const;
 
   const DictionaryType* dict_type() const { return dict_type_; }
 
-  std::shared_ptr<Array> Slice(int64_t offset, int64_t length) const override;
-
  private:
-  void SetData(const std::shared_ptr<internal::ArrayData>& data);
+  void SetData(const std::shared_ptr<ArrayData>& data);
 
   const DictionaryType* dict_type_;
   std::shared_ptr<Array> indices_;
@@ -695,18 +670,27 @@ ARROW_EXTERN_TEMPLATE NumericArray<TimestampType>;
 ///
 /// \param array an Array instance
 /// \return Status
-Status ARROW_EXPORT ValidateArray(const Array& array);
+ARROW_EXPORT
+Status ValidateArray(const Array& array);
+
+#ifndef ARROW_NO_DEPRECATED_API
+// \deprecated Since 0.7.0
 
 /// Create new arrays for logical types that are backed by primitive arrays.
-Status ARROW_EXPORT MakePrimitiveArray(const std::shared_ptr<DataType>& type,
-    int64_t length, const std::shared_ptr<Buffer>& data,
-    const std::shared_ptr<Buffer>& null_bitmap, int64_t null_count, int64_t offset,
-    std::shared_ptr<Array>* out);
+ARROW_EXPORT
+Status MakePrimitiveArray(const std::shared_ptr<DataType>& type, int64_t length,
+                          const std::shared_ptr<Buffer>& data,
+                          const std::shared_ptr<Buffer>& null_bitmap, int64_t null_count,
+                          int64_t offset, std::shared_ptr<Array>* out);
 
-Status ARROW_EXPORT MakePrimitiveArray(const std::shared_ptr<DataType>& type,
-    const std::vector<std::shared_ptr<Buffer>>& buffers, int64_t length,
-    int64_t null_count, int64_t offset, std::shared_ptr<Array>* out);
+ARROW_EXPORT
+Status MakePrimitiveArray(const std::shared_ptr<DataType>& type,
+                          const std::vector<std::shared_ptr<Buffer>>& buffers,
+                          int64_t length, int64_t null_count, int64_t offset,
+                          std::shared_ptr<Array>* out);
+
+#endif
 
 }  // namespace arrow
 
-#endif
+#endif  // ARROW_ARRAY_H
